@@ -5,7 +5,7 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2015-2023 Statnet Commons
+#  Copyright 2015-2024 Statnet Commons
 ################################################################################
 ## A thin wrapper around summary.ergm to get rid of a spurious error message.
 #' @method summary ergm.ego
@@ -27,3 +27,9 @@ print.summary.ergm.ego <- function (x, ...){
   NextMethod("print", object=x, ..., print.deviances=FALSE)
 }
 
+#' @method logLik ergm.ego
+#' @export
+#' @noRd
+logLik.ergm.ego <- function(object, ...){
+  stop("Log-likelihood is not meaningful for moments-based inference used by ", sQuote("ergm.ego()"), ".")
+}
